%SIMULAZIONE DEL POTENZIALE DI AZIONE 
%condizioni: assone gigante di calamaro T=6.3?C

clear all
%close all
clc

%% costanti

GNaM=120; %[mS/cm2]
GKM=36; %[mS/cm2]
GL=0.3; %[mS/cm2]
CM=1; %[uF/cm2]
Vr=-65; %[mv]
VNa=50; %[mv]
VK=-77; %[mv]
VL=-54.387; %[mv]

%% Temperatura
T=36.5;

%% calcolo e grafico dei coefficienti di velocit? di transizione [1/ms] e delle conduttanze al variare di V

v=-100:0.1:40; % intervallo variazione potenziale di membrana
dv=v-Vr; %v-Vr=V

%inizializzo le variabili
a_n=zeros(size(v));
b_n=zeros(size(v));
a_m=zeros(size(v));
b_m=zeros(size(v));
a_h=zeros(size(v));
b_h=zeros(size(v));


for i=1:length(v)
a_n(i)=(0.01*(10-(dv(i))))/(exp((10-dv(i))/10)-1); 
b_n(i)=0.125/(exp(dv(i)/80));
a_m(i)=(0.1*(25-(dv(i))))/(exp((25-dv(i))/10)-1); 
b_m(i)=4/(exp(dv(i)/18));
a_h(i)=0.07/(exp(dv(i)/20));
b_h(i)=1/(exp((30-dv(i))/10)+1); 
end

%coefficienti v trans n al variare di v
figure
plot(v,a_n,'b');
hold on
plot(v,b_n,'r');
xlabel('v [mv]');
legend('\alpha_n','\beta_n');


%calcolo valore di regime al variare di V
n_inf=a_n./(a_n+b_n);
figure
plot(v,n_inf,'b');
hold on
plot(v,n_inf.^4,'r');
xlabel('v [mv]');
legend('n_{inf}','n_{inf}^4');

%plotto Gk al variare di V
figure
plot(v,GKM*(n_inf.^4));
xlabel('v [mv]');
legend('G_K');

%coefficienti v trans m al variare di v
figure
plot(v,a_m,'b');
hold on
plot(v,b_m,'r');
xlabel('v [mv]');
legend('\alpha_m','\beta_m');

%coefficienti v trans h al variare di v
figure
plot(v,a_h,'b');
hold on
plot(v,b_h,'r');
xlabel('v [mv]');
legend('\alpha_h','\beta_h');



%calcolo valore di regime al variare di V
m_inf=a_m./(a_m+b_m);
h_inf=a_h./(a_h+b_h);
figure
subplot(2,1,1)
plot(v,m_inf,'b');
hold on
plot(v,h_inf,'r');
xlabel('v [mv]');
legend('m_{inf}','h_{inf}');
subplot(2,1,2);
plot(v,(m_inf.^3).*h_inf);
xlabel('v [mv]');
legend('m_{inf}^3 h_{inf}');


figure
plot(v,GNaM*((m_inf.^3).*h_inf));
xlabel('v [mv]');
legend('G_{Na}');

%% condizioni iniziali : le calcolo come alpha(V=Vr)/(alpha(V=Vr)+beta(V=Vr))

%condizioni iniziale n_inf(V=Vr) h_inf(V=Vr)  m_inf(V=Vr) 
V0=Vr;
ind_r=find(v==V0);
%le condizioni iniziali per n,m,h dipendono dalla tensione
n0=n_inf(ind_r); %0.3177;
m0=m_inf(ind_r); %0.0529;
h0=h_inf(ind_r); %0.5961;
%% variabili di integrazione

tmax=10; % [msec] NB: i coefficienti di vel di transizione sono in 1/msec!
dt=0.0001;% [ms]
steps=round(tmax/dt);
t=(0:steps-1)*dt;
%% inizializzo le variabili con la giusta dimensione 

  Vm = zeros(steps,1);
  m = zeros(steps,1);
  h = zeros(steps,1);
  n = zeros(steps,1);
  GK=zeros(steps,1);
  GNa=zeros(steps,1);
  GTot=zeros(steps,1);
  IK=zeros(steps,1);
  INa=zeros(steps,1);
  IL=zeros(steps,1);
  Ii=zeros(steps,1);
  I=zeros(steps,1);
  
 
%%  Corrente di stimolazione

  stimolo=20e-9; %C/cm2 %soglia 6.50 
  durata=0.1; %msec
  IS=(stimolo)/(durata*1e-3); %corrente in A
  ncampioni=round(durata/dt);
  %ii=(12e-9)/(2e-3);
  %ncampioni=2/dt;
  IS=IS*10^6;% [uA/cm2]
  I(ncampioni:ncampioni+ncampioni)=IS;
  figure
  plot(t,I);
  
%% refrattario (calcoli fatti per stimolo di 20e-9)
  %N=30; %8-37 assoluto;  N=60 o 80  realativo il solito stimolo di prima non basta, uno pi? alto si (4Is) (Is) non ce la fa nel relativo h deve essere sufficientemente alto, vedi figura finale
  %I(N*ncampioni:N*ncampioni+ncampioni)=4*IS; %REFRATTARIO: con  stimolo (Riperto lo stimolo mentre la membrana ? attivata)
  
%% firing periodico
  %I(1:end)=10; %per firing periodico -> piccola corrente costante
  
%%  setto le condizioni iniziali

  
  Vm(1)=V0;
  m(1)=m0;
  n(1)=n0;
  h(1)=h0;
  
%% Metodo Eulero: soluzione di Im=Ic+Ii -> impulso non propagante (space clamp) stimolato con una corrente esterna
%   

for i=1:steps-1
      
      %conduttanze
      GK(i)=GKM*(n(i)^4);
      GNa(i)=GNaM*(m(i)^3)*h(i);
      GTot(i)=GL+GK(i)+GNa(i);
      
      %correnti
      IK(i)=GK(i)*(Vm(i)-VK); %K
      INa(i)=GNa(i)*(Vm(i)-VNa); %Na
      IL(i)=GL*(Vm(i)-VL); %L
      Ii(i)=IK(i)+INa(i)+ IL(i);
            
      Vm(i+1)=Vm(i)+dt*( I(i)- Ii(i))/CM;
      
      
      dv=Vm(i)-Vr;
      
      an=(0.01*(10-(dv)))/(exp((10-dv)/10)-1);
      bn=0.125/(exp(dv/80));
      am=(0.1*(25-(dv)))/(exp((25-dv)/10)-1); 
      bm=4/(exp(dv/18));
      ah=0.07/(exp(dv/20));
      bh=1/(exp((30-dv)/10)+1); 

      
%       n(i+1)= n(i)+dt*(an*(1-n(i))-bn*n(i));
%       m(i+1)= m(i)+dt*(am*(1-m(i))-bm*m(i));
%       h(i+1)= h(i)+dt*(ah*(1-h(i))-bh*h(i));
      
      %SE CONSIDERO DIPENDENZA DA T
      n(i+1)= n(i)+dt*(  (an*(1-n(i))-bn*n(i))*3^((T-6.3)/10)   ); 
      m(i+1)= m(i)+dt*( (am*(1-m(i))-bm*m(i))*3^((T-6.3)/10)   ); 
      h(i+1)= h(i)+dt*( (ah*(1-h(i))-bh*h(i))*3^((T-6.3)/10) ); 
      
end
  
 
%% Grafico la corrente di stimolo e il potenziale di azione

figure
plot(t,I);
xlabel('t [msec]');
legend('IS');
ylabel('[\mu A/cm^2]'); %mS*mV

figure
plot(t,Vm);
ylabel('[mV]');
xlabel('t [msec]');
hold on
plot(t,ones(size(Vm))*Vr,'-.');
legend('Vm','Vr');
ylim([-90 40]);




% con corrente
% figure
% subplot(2,1,1)
% plot(t,Vm);
% legend('Vm');
% ylabel('[mV]');
% xlabel('t [msec]');
% subplot(2,1,2)
% plot(t,I);
% legend('I');
% xlabel('t [msec]');
% ylabel('[\mu A/cm^2]'); %mS*mV
%axis([t(1) t(length(t)) -70 20]);

%% Grafico  conduttanze

figure
plot(t,GK);
hold on
plot(t,GNa);
plot(t,GTot,'g');
scaleV=40/100;
plot(t,(Vm-Vr)*scaleV,'k--');
legend('GK','GNa','Gtot','\Delta Vm');
xlabel('t [msec]');
ylabel('mS/cm^2');



%% refrattario

%  figure
%  subplot(2,1,1)
%  plot(t,h)
%  legend('h')
%  xlabel('t [msec]');
%  subplot(2,1,2)
%  plot(t,I)
%  xlabel('t [msec]');